import unittest
from mock import patch, MagicMock
import mock
try:
    from mock import call
except ImportError:
    pass
from dputrunner import DputRunner
import subprocess
from distutils.version import StrictVersion
from testscenarios import TestWithScenarios


class TestDputBase(unittest.TestCase):

    def setUp(self):
        super(TestDputBase, self).setUp()

        self.ppa = 'ppa:test-team/ppa'
        self.trunk = 'lp:dummy'
        self.packagingBranch = 'lp:ubuntu/dummy'
        self.DEBFULLNAME = 'ps-jenkins'
        self.DEBEMAIL = 'ps-jenkins@'

    def make_dput_runner(self, **kwargs):
        """Make an instance of dputRunner, returning it and setting it at
        self.dput_runner.

        Keyword arguments can be passed in to replace the defaults.

        """
        keyword_args = dict(ppas=[self.ppa],
            trunk=self.trunk,
            packagingBranch=self.packagingBranch,
            DEBFULLNAME=self.DEBFULLNAME,
            DEBEMAIL=self.DEBEMAIL)
        keyword_args.update(kwargs)
        self.dput_runner = DputRunner(**keyword_args)
        return self.dput_runner


class TestDputRunner(TestDputBase):

    def setUp(self):
        super(TestDputRunner, self).setUp()
        self.ppas = ['ppa:team/staging', 'ppa:team/release']
        self.distributions = ['precise', 'quantal']
        self.make_dput_runner(ppas=self.ppas, distributions=self.distributions)
        self.dput_runner.formatVersionString = MagicMock(return_value='3')
        self.dput_runner.doDput = MagicMock()

    @unittest.skipIf(StrictVersion(mock.__version__) < StrictVersion('0.8'),
                     "assert_has_calls not suppored by mock < 0.8")
    def test_dput(self):
        recipe = MagicMock()
        recipe.getTrunkRevno = lambda: 10
        with patch('dputrunner.BzrRecipe', new=MagicMock(return_value=recipe)):
            self.dput_runner.dput()
            calls = []
            calls.append(call(recipe, '3', "Automatic build of revision 10",
                               'precise', 'ppa:team/staging'))
            calls.append(call(recipe, '3', "Automatic build of revision 10",
                               'precise', 'ppa:team/release'))
            calls.append(call(recipe, '3', "Automatic build of revision 10",
                               'quantal', 'ppa:team/staging'))
            calls.append(call(recipe, '3', "Automatic build of revision 10",
                               'quantal', 'ppa:team/release'))
            self.dput_runner.doDput.assert_has_calls(calls)


class TestDput(TestDputBase):

    def setUp(self):
        super(TestDput, self).setUp()
        self.make_dput_runner()

    def test_doDput_called_process_error(self):
        check_call = MagicMock()
        check_call.side_effect = subprocess.CalledProcessError(3, ['bash'])
        with patch('dputrunner.subprocess.check_call', check_call):
            self.assertRaises(subprocess.CalledProcessError,
                              self.dput_runner.doDput, MagicMock(),
                                                     'version1.1',
                                                     'Changelog',
                                                     'quantal',
                                                     'ppa:team/staging')

    @unittest.skipIf(StrictVersion(mock.__version__) < StrictVersion('0.8'),
                     "assert_has_calls not suppored by mock < 0.8")
    def test_doDput(self):
        recipe = MagicMock()
        recipe.getTrunkDir = MagicMock(return_value='/trunk')
        recipe.getSourceName = MagicMock(return_value='sourceName')
        recipe.getBuildDir = MagicMock(return_value='/build')
        check_call = MagicMock()
        version_string = 'version1.1'
        changelog_string = 'Changelog'
        distribution_string = 'quantal'
        ppa = 'ppa:team/staging'

        with patch('dputrunner.subprocess.check_call', check_call):
            self.dput_runner.doDput(recipe, version_string, changelog_string,
                                    distribution_string, ppa)
            calls = []
            calls.append(call(
                ['/usr/share/pbuilderjenkins/hooks/H40native_hack.py'],
                cwd='/trunk'))
            calls.append(call(['dch', '-v', version_string, changelog_string,
                              '-D', distribution_string], cwd='/trunk'))
            calls.append(call(['debuild', '--no-tgz-check', '-S', '-d'],
                              cwd='/trunk'))
            calls.append(call(['dput', ppa, 'sourceName_' +
                               version_string + '_source.changes'],
                              cwd='/build'))
            calls.append(call(['rm', '-f', 'sourceName_' +
                               version_string + '_source.ppa.upload'],
                              cwd='/build'))

            check_call.assert_has_calls(calls)


class DPutVersionTests(TestWithScenarios, TestDput):
    """A nscenariod test suite that tests DPut version string formatting."""

    scenarios = [
        ('simple version',
            {
                'formatString': '{PACKAGE_VERSION}',
                'variables': {
                    'PACKAGE_VERSION': 13,
                    'BZR_REVISION': 0,
                    'PACKAGING_REVISION': 0,
                    'JENKINS_BUILD_NUMBER': 0,
                    'DISTRIBUTION': 'precise',
                },
                'expectedResult': '13'
            }),
        ('version with escaped tag',
            {
                'formatString': '{PACKAGE_VERSION}{{I_WANT_THIS}}',
                'variables': {
                    'PACKAGE_VERSION': 13,
                    'BZR_REVISION': 0,
                    'PACKAGING_REVISION': 0,
                    'JENKINS_BUILD_NUMBER': 0,
                    'DISTRIBUTION': 'precise',
                },
                'expectedResult': '13{I_WANT_THIS}'
            }),
        ('no format string',
            {
                'formatString': None,
                'variables':  {
                    'PACKAGE_VERSION': 13,
                    'BZR_REVISION': 43,
                    'PACKAGING_REVISION': 7,
                    'JENKINS_BUILD_NUMBER': 0,
                    'DISTRIBUTION': 'precise',
                },
                'expectedResult': '13+bzr43+pkg7~precise1'
            }),
        ('complex format string',
            {
                'formatString':
                                 '{PACKAGE_VERSION}-{BZR_REVISION}-' +
                                 '{PACKAGING_REVISION}-' +
                                 '{JENKINS_BUILD_NUMBER}-' +
                                 '{DISTRIBUTION}',
                'variables': {
                    'PACKAGE_VERSION': 13,
                    'BZR_REVISION': 43,
                    'PACKAGING_REVISION': 7,
                    'JENKINS_BUILD_NUMBER': 1234,
                    'DISTRIBUTION': 'precise',
                },
                'expectedResult': '13-43-7-1234-precise'
            }),
        ('unknown tag',
            {
                'formatString':
                                 '{PACKAGE_VERSION}-{UNKOWN_TAG}',
                'variables': {
                    'PACKAGE_VERSION': 13,
                    'BZR_REVISION': 43,
                    'PACKAGING_REVISION': 7,
                    'JENKINS_BUILD_NUMBER': 1234,
                    'DISTRIBUTION': 'precise',
                },
                'expectedResult': None
            }),
        ]

    def test_formatVersionString(self):

        self.make_dput_runner(versionFormat=self.formatString)

        result_format = self.dput_runner.formatVersionString(**self.variables)
        self.assertEquals(self.expectedResult, result_format)
